/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.rest;

import java.net.URI;
import java.util.List;

import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.rest.RestAgreement;
import org.ogf.graap.wsag.server.persistence.PersistentAgreement;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;
import org.ogf.schemas.graap.wsAgreement.AgreementContextType;
import org.ogf.schemas.graap.wsAgreement.AgreementPropertiesDocument;
import org.ogf.schemas.graap.wsAgreement.AgreementPropertiesType;
import org.ogf.schemas.graap.wsAgreement.AgreementStateType;
import org.ogf.schemas.graap.wsAgreement.GuaranteeTermStateType;
import org.ogf.schemas.graap.wsAgreement.ServiceTermStateType;
import org.ogf.schemas.graap.wsAgreement.TermTreeType;
import org.ogf.schemas.graap.wsAgreement.TerminateInputDocument;

/**
 * @author owaeld
 * 
 */
public class RestAgreementFacade
    implements RestAgreement
{
    private final PersistentAgreement agreement;

    private final PersistentAgreementFactory agreementFactory;

    public RestAgreementFacade( PersistentAgreement agreement, PersistentAgreementFactory agreementFactory )
    {
        this.agreement = agreement;
        this.agreementFactory = agreementFactory;
    }

    private PersistentAgreement getAgreement()
    {
        //
        // return the agreement instance
        //
        return agreement;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getResourceProperties()
     */
    @Override
    public AgreementPropertiesDocument getResourceProperties()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementPropertiesDocument document = AgreementPropertiesDocument.Factory.newInstance();
        AgreementPropertiesType properties = getAgreement().getAgreement().getXMLObject();
        document.addNewAgreementProperties().set( properties );
        return document;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getContext()
     */
    @Override
    public AgreementPropertiesDocument getContext()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementContextType context = getAgreement().getAgreement().getContext();
        AgreementPropertiesDocument properties = AgreementPropertiesDocument.Factory.newInstance();
        properties.addNewAgreementProperties().setContext( context );
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getGuaranteeTermStates()
     */
    @Override
    public AgreementPropertiesDocument getGuaranteeTermStates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        GuaranteeTermStateType[] guaranteeTermStates = getAgreement().getAgreement().getGuaranteeTermStates();
        AgreementPropertiesDocument properties = AgreementPropertiesDocument.Factory.newInstance();
        properties.addNewAgreementProperties().setGuaranteeTermStateArray( guaranteeTermStates );
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getAgreementId()
     */
    @Override
    public String getAgreementId() throws ResourceUnknownException, ResourceUnavailableException
    {
        return getAgreement().getAgreement().getAgreementId();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getName()
     */
    @Override
    public String getName() throws ResourceUnknownException, ResourceUnavailableException
    {
        return getAgreement().getAgreement().getName();
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getServiceTermStates()
     */
    @Override
    public AgreementPropertiesDocument getServiceTermStates()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        ServiceTermStateType[] serviceTermStates = getAgreement().getAgreement().getServiceTermStates();
        AgreementPropertiesDocument properties = AgreementPropertiesDocument.Factory.newInstance();
        properties.addNewAgreementProperties().setServiceTermStateArray( serviceTermStates );
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getState()
     */
    @Override
    public AgreementPropertiesDocument getState()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        AgreementStateType state = getAgreement().getAgreement().getState();
        AgreementPropertiesDocument properties = AgreementPropertiesDocument.Factory.newInstance();
        properties.addNewAgreementProperties().setAgreementState( state );
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getTerms()
     */
    @Override
    public AgreementPropertiesDocument getTerms()
        throws ResourceUnknownException, ResourceUnavailableException
    {
        TermTreeType terms = getAgreement().getAgreement().getTerms();
        AgreementPropertiesDocument properties = AgreementPropertiesDocument.Factory.newInstance();
        properties.addNewAgreementProperties().setTerms( terms );
        return properties;
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#getOperations()
     */
    @Override
    public List<URI> getOperations() throws ResourceUnknownException, ResourceUnavailableException
    {
        throw new UnsupportedOperationException( "not yet implemented" );
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#terminate(org.ogf.schemas.graap.wsAgreement.
     * TerminateInputDocument)
     */
    @Override
    public void terminate( TerminateInputDocument reason )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        getAgreement().getAgreement().terminate( reason.getTerminateInput() );
    }

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreement#destroy()
     */
    @Override
    public void destroy() throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            PersistentAgreement agreement = getAgreement();
            getFactory().remove( agreement.getAgreement().getAgreementId() );
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( "Failed to remove agreement instance.", e );
        }
    }

    /**
     * @return
     */
    private PersistentAgreementFactory getFactory()
    {
        return agreementFactory;
    }

}
