/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.rest;

import java.net.URI;
import java.text.MessageFormat;
import java.util.List;
import java.util.Vector;

import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.UriBuilder;
import javax.ws.rs.core.UriInfo;

import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.rest.RestAgreementFactory;
import org.ogf.graap.wsag.api.rest.RestAgreementFactoryRegistry;
import org.ogf.graap.wsag.server.persistence.PersistentAgreementFactory;

/**
 * @author owaeld
 * 
 */
public class RestAgreementFactoryRegistryFacade
    implements RestAgreementFactoryRegistry
{
    @Context
    private UriInfo uriInfo;

    /*
     * (non-Javadoc)
     * 
     * @see org.ogf.graap.wsag.api.rest.RestAgreementFactoryRegistry#listFactoryReferences()
     */
    @Override
    public List<URI> listFactoryReferences() throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            PersistentAgreementFactory[] factories = RestWsagEngine.getInstance().list();

            List<URI> uris = new Vector<URI>();
            for ( int i = 0; i < factories.length; i++ )
            {
                String resourceId = factories[i].getResourceId();

                URI factoryUri = getUriBuilder().path( resourceId ).build();

                uris.add( factoryUri );
            }

            return uris;
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( e );
        }
    }

    @Path( "/{factoryId}" )
    public RestAgreementFactory getFactory( @PathParam( "factoryId" ) String factoryId )
        throws ResourceUnknownException, ResourceUnavailableException
    {
        try
        {
            //
            // lookup the factory with the given id from the WSAG4J engine
            //
            PersistentAgreementFactory factoryInstance = RestWsagEngine.getInstance().find( factoryId );

            //
            // if the engine returns null, we raise a resource not found exception
            //
            if ( factoryInstance == null )
            {
                String message =
                    MessageFormat.format( "Factory resource with id ''{0}'' is unknown.", factoryId );

                throw new ResourceUnknownException( message );
            }

            return new RestAgreementFactoryFacade( factoryId, factoryInstance, uriInfo );
        }

        catch ( ResourceUnknownException e )
        {
            throw e;
        }
        catch ( Exception e )
        {
            throw new ResourceUnavailableException( "failed to lookup factory resource", e );
        }
    }

    private UriBuilder getUriBuilder()
    {
        return uriInfo.getAbsolutePathBuilder();
    }
}
