/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.rest;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

import javax.ws.rs.Path;

import org.ogf.graap.wsag.api.Negotiation;

/**
 * Helper class to store and lookup negotiation instances under an unique id. This class will be removed once
 * the registry negotiation capability is implemented in the WSAG4J engine.
 * 
 * @author owaeld
 * 
 */
@Path( "/factories/{factoryId}/Negotiations" )
public class RestNegotiationRegistry
{
    protected static final Map<String, Negotiation> negotiations = new HashMap<String, Negotiation>();

    /**
     * Adds a negotiation object to the registry and returns a unique id for this object.
     * 
     * @param negotiation
     *            the negotiation to add
     * 
     * @return the unique id
     */
    public static synchronized String add( Negotiation negotiation )
    {
        String id = UUID.randomUUID().toString();
        while ( negotiations.containsKey( id ) )
        {
            id = UUID.randomUUID().toString();
        }
        negotiations.put( id, negotiation );
        return id;
    }

    /**
     * Retrieves the negotiation object stored with the provided key. if the key does not exist in the
     * registry this method returns null.
     * 
     * @param id
     *            the negotiation id
     * 
     * @return the referenced negotiation object
     */
    public static Negotiation get( String id )
    {
        return negotiations.get( id );
    }

    /**
     * Removes the negotiation object with the given key from the registry and returns the removed object. If
     * no negotiation exists with the given key this method returns null.
     * 
     * @param id
     *            the negotiation id
     */
    public static synchronized Negotiation remove( String id )
    {
        return negotiations.remove( id );
    }
}
