/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.rest;

import org.ogf.graap.wsag.server.engine.EngineInstantiationException;
import org.ogf.graap.wsag.server.engine.WsagEngine;

/**
 * @author owaeld
 * 
 */
public class RestWsagEngine
{
    /**
     * The name of the default configuration file for the REST remote stack. The configuration file must exist
     * in the classpath.
     */
    public static final String WSAG4J_ENGINE_CONFIG_FILE = "/wsag4j-engine.config";

    private static WsagEngine engine = null;

    private RestWsagEngine()
    {
        super();
    }

    /**
     * Returns the {@link WsagEngine} instance used by the REST remote stack.
     * 
     * @return the engine instance
     */
    public static synchronized WsagEngine getInstance()
    {
        return engine;
    }

    /**
     * Creates a new {@link WsagEngine} instance and initializes the instance using the default configuration
     * file.
     * 
     * @throws EngineInstantiationException
     *             failed to startup default engine
     */
    public static synchronized void startup() throws EngineInstantiationException
    {
        if ( engine != null )
        {
            return;
        }
        engine = WsagEngine.getInstance( WSAG4J_ENGINE_CONFIG_FILE );
    }

    /**
     * Terminates the default {@link WsagEngine} instance gracefully.
     * 
     * @throws Exception
     *             error during shutdown procedure
     */
    public static synchronized void shutdown() throws Exception
    {
        engine.shutdown();
        engine = null;
    }

}
