/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.rest.acceptance;

import java.text.MessageFormat;

import org.apache.velocity.exception.ResourceNotFoundException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.api.exceptions.ResourceUnknownException;
import org.ogf.graap.wsag.api.rest.RestAcceptanceEndpoint;
import org.ogf.graap.wsag.server.api.AgreementAcceptanceListener;
import org.ogf.schemas.graap.wsAgreement.AcceptAgreementInputDocument;
import org.ogf.schemas.graap.wsAgreement.RejectAgreementInputDocument;

/**
 * @author owaeld
 * 
 */
public class RestAcceptanceEndpointFacade
    implements RestAcceptanceEndpoint
{
    private final String acceptanceId;

    private final AgreementAcceptanceListener listener;

    public RestAcceptanceEndpointFacade( String acceptanceId, AgreementAcceptanceListener listener )
    {
        this.acceptanceId = acceptanceId;
        this.listener = listener;
    }

    @Override
    public void reject( RejectAgreementInputDocument input )
        throws ResourceNotFoundException, ResourceUnavailableException
    {
        try
        {
            listener.reject();
        }
        catch ( Exception e )
        {
            String message =
                MessageFormat.format( "the acceptance endpoint ''{0}'' is unavailable", acceptanceId );
            throw new ResourceUnavailableException( message, e );
        }
    }

    @Override
    public void accept( AcceptAgreementInputDocument input )
        throws ResourceNotFoundException, ResourceUnavailableException
    {
        try
        {
            listener.accept();
        }
        catch ( Exception e )
        {
            String message =
                MessageFormat.format( "the acceptance endpoint ''{0}'' is unavailable", acceptanceId );
            throw new ResourceUnavailableException( message, e );
        }
    }

    @Override
    public void destroy() throws ResourceUnknownException, ResourceUnavailableException
    {
        boolean available = RestAgreementAcceptanceRegistry.getInstance().containsKey( acceptanceId );
        if ( !available )
        {
            String message =
                MessageFormat.format( "the acceptance endpoint ''{0}'' is unknown", acceptanceId );
            throw new ResourceUnknownException( message );
        }

        AgreementAcceptanceListener removed =
            RestAgreementAcceptanceRegistry.getInstance().remove( acceptanceId );

        if ( removed == null )
        {
            String message =
                MessageFormat.format( "the acceptance endpoint ''{0}'' is unknown", acceptanceId );
            throw new ResourceUnavailableException( message );
        }
    }
}
