/* 
 * Copyright (c) 2012, Fraunhofer-Gesellschaft
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * (1) Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the disclaimer at the end.
 *     Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 * 
 * (2) Neither the name of Fraunhofer nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 * 
 * DISCLAIMER
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */
package org.ogf.graap.wsag.server.rest.acceptance;

import javax.ws.rs.core.UriBuilder;

import org.ogf.graap.wsag.api.exceptions.ResourceUnavailableException;
import org.ogf.graap.wsag.server.api.AgreementAcceptanceFactory;
import org.ogf.graap.wsag.server.api.AgreementAcceptanceListener;
import org.w3.x2005.x08.addressing.EndpointReferenceType;

/**
 * @author owaeld
 * 
 */
public class RestAgreementAcceptanceFactory
    implements AgreementAcceptanceFactory
{
    private final UriBuilder acceptanceURIBuilder;

    /**
     * 
     */
    public RestAgreementAcceptanceFactory( UriBuilder acceptanceURI )
    {
        this.acceptanceURIBuilder = acceptanceURI;
    }

    /**
     * default constructor
     */
    public static RestAgreementAcceptanceFactory newInstance( UriBuilder factoryURI )
        throws ResourceUnavailableException
    {
        //
        //
        // if ( ( factoryURI == null ) || "".equals( factoryURI ) )
        // {
        // throw new RuntimeException( "factory uri not found in context" );
        // }
        //
        // int factoryUriStart = factoryURI.indexOf( RestConstants.REST_FACTORY_REGISTRY_PATH + "/" );
        // if ( factoryUriStart <= 0 )
        // {
        // throw new ResourceUnavailableException(
        // "Invalid factory uri. Failed to build acceptance uri for factory uri." );
        // }
        //
        // String uri = factoryURI.substring( 0, factoryUriStart );
        return new RestAgreementAcceptanceFactory( factoryURI );
    }

    /*
     * (non-Javadoc)
     * 
     * @see
     * org.ogf.graap.wsag.server.api.AgreementAcceptanceFactory#registerAgreementAccetanceListener(org.ogf
     * .graap.wsag.server.api.AgreementAcceptanceListener)
     */
    @Override
    public EndpointReferenceType registerAgreementAccetanceListener( AgreementAcceptanceListener handler )
    {
        String id =
            RestAgreementAcceptanceRegistry.getInstance().registerAgreementAccetanceListener( handler );

        EndpointReferenceType epr = EndpointReferenceType.Factory.newInstance();
        String uri = acceptanceURIBuilder.path( id ).build().toASCIIString();
        epr.addNewAddress().setStringValue( uri );
        return epr;
    }

}
